@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2017 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,6 +48,7 @@ typedef struct
 	MsgInfo *msginfo;
 	gint n_pages;
 	gchar *hdr_data;
+	gchar *msg_text_file;
 	FILE *fp;
 } MsgPrintInfo;
 
@@ -170,11 +171,10 @@ static gint message_count_page(MsgPrintInfo *mpinfo, GtkPrintContext *context,
 	PangoFontDescription *desc;
 	gint layout_h;
 	gint lines_per_page, lines_left;
-	gint body_lines = 0;
 	gint n_pages = 1;
 	PageInfo *pinfo;
 	gint i;
-	FILE *fp;
+	FILE *fp = NULL;
 	gchar buf[BUFFSIZE];
 	glong pos = 0;
 
@@ -218,8 +218,10 @@ static gint message_count_page(MsgPrintInfo *mpinfo, GtkPrintContext *context,
 		body_h = 0.0;
 	lines_left = body_h / line_h;
 
-	g_print("layout_h = %d, line_h = %g, lines_per_page = %d\n", layout_h, line_h, lines_per_page);
-	g_print("hdr_h = %g, body_h = %g, lines_left = %d\n", hdr_h, body_h, lines_left);
+	debug_print("width = %g, height = %g\n", width, height);
+	debug_print("dpi_x = %g, dpi_y = %g\n", gtk_print_context_get_dpi_x(context), gtk_print_context_get_dpi_y(context));
+	debug_print("layout_h = %d, line_h = %g, lines_per_page = %d\n", layout_h, line_h, lines_per_page);
+	debug_print("hdr_h = %g, body_h = %g, lines_left = %d\n", hdr_h, body_h, lines_left);
 
 	if (print_data->partinfo) {
 		FILE *msgfp;
@@ -230,7 +232,14 @@ static gint message_count_page(MsgPrintInfo *mpinfo, GtkPrintContext *context,
 					       NULL);
 		fclose(msgfp);
 	} else {
-		fp = procmime_get_first_text_content(mpinfo->msginfo, NULL);
+		mpinfo->msg_text_file = get_tmp_file();
+		if (procmsg_save_message_as_text(mpinfo->msginfo, mpinfo->msg_text_file, NULL, print_data->all_headers) == 0) {
+			if ((fp = g_fopen(mpinfo->msg_text_file, "rb")) != NULL) {
+				while (fgets(buf, sizeof(buf), fp) != NULL)
+					if (buf[0] == '\r' || buf[0] == '\n')
+						break;
+			}
+		}
 	}
 	if (!fp) {
 		g_warning("Can't get text part\n");
@@ -257,7 +266,7 @@ static gint message_count_page(MsgPrintInfo *mpinfo, GtkPrintContext *context,
 		while (lines_left < lines) {
 			PangoLayoutLine *line;
 
-			g_print("page increment: %d: lines_left = %d, lines = %d\n", i, lines_left, lines);
+			debug_print("page increment: %d: lines_left = %d, lines = %d\n", i, lines_left, lines);
 			line_offset += lines_left;
 			line = pango_layout_get_line(layout, line_offset);
 			lines -= lines_left;
@@ -281,7 +290,7 @@ static gint message_count_page(MsgPrintInfo *mpinfo, GtkPrintContext *context,
 	g_object_unref(layout);
 
 	n_pages = i + 1;
-	g_print("n_pages = %d\n", n_pages);
+	debug_print("n_pages = %d\n", n_pages);
 
 	return n_pages;
 }
@@ -290,8 +299,6 @@ static void begin_print(GtkPrintOperation *operation, GtkPrintContext *context,
 			gpointer data)
 {
 	PrintData *print_data = data;
-	GSList *cur;
-	MsgInfo *msginfo;
 	gint n_pages = 0;
 	gint i;
 
@@ -379,7 +386,6 @@ static void draw_page(GtkPrintOperation *operation, GtkPrintContext *context,
 		PangoLayoutIter *iter;
 		PangoLayoutLine *layout_line;
 		gint baseline;
-		gint y0, y1;
 		gdouble x, y;
 
 		if (fgets(buf, sizeof(buf), mpinfo->fp) == NULL)
@@ -409,7 +415,7 @@ static void draw_page(GtkPrintOperation *operation, GtkPrintContext *context,
 		cairo_move_to(cr, 0,
 			      y + (gdouble)layout_h / PANGO_SCALE + SPACING);
 	}
-	g_print("count = %d\n", count);
+	debug_print("count = %d\n", count);
 
 	desc = gtkut_get_default_font_desc();
 	pango_font_description_set_size(desc, font_size);
@@ -424,6 +430,116 @@ static void draw_page(GtkPrintOperation *operation, GtkPrintContext *context,
 	g_object_unref(layout);
 }
 
+static gboolean printing_load_settings(void)
+{
+#if GTK_CHECK_VERSION(2, 12, 0)
+	gchar *file;
+	GtkPrintSettings *new_settings;
+
+	debug_print("printing_load_settings: load settings\n");
+
+	file = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, "printingrc", NULL);
+	if (!is_file_exist(file)) {
+		debug_print("printing_load_settings: %s not exist\n", file);
+		g_free(file);
+		return FALSE;
+	}
+
+	if ((new_settings = gtk_print_settings_new_from_file(file, NULL)) == NULL) {
+		g_warning("printing_load_settings: gtk_print_settings_new_from_file: %s: error", file);
+		g_free(file);
+		return FALSE;
+	}
+	g_free(file);
+
+	if (settings)
+		g_object_unref(settings);
+	settings = new_settings;
+
+	return TRUE;
+#else
+	return FALSE;
+#endif /* GTK_CHECK_VERSION(2, 12, 0) */
+}
+
+static gboolean printing_save_settings(void)
+{
+#if GTK_CHECK_VERSION(2, 12, 0)
+	gchar *file;
+	gboolean ret = TRUE;
+
+	g_return_val_if_fail(settings != NULL, FALSE);
+
+	debug_print("printing_save_settings: save settings\n");
+
+	file = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, "printingrc", NULL);
+	if (gtk_print_settings_to_file(settings, file, NULL) != TRUE) {
+		g_warning("printing_save_settings: gtk_print_settings_to_file: %s: error", file);
+		ret = FALSE;
+	}
+	g_free(file);
+
+	return ret;
+#else
+	return FALSE;
+#endif /* GTK_CHECK_VERSION(2, 12, 0) */
+}
+
+static gboolean printing_load_page_setup(void)
+{
+#if GTK_CHECK_VERSION(2, 12, 0)
+	gchar *file;
+	GtkPageSetup *new_setup;
+
+	debug_print("printing_load_page_setup: load page setup\n");
+
+	file = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, "pagesetuprc", NULL);
+	if (!is_file_exist(file)) {
+		debug_print("printing_load_page_setup: %s not exist\n", file);
+		g_free(file);
+		return FALSE;
+	}
+
+	if ((new_setup = gtk_page_setup_new_from_file(file, NULL)) == NULL) {
+		g_warning("printing_load_page_setup: gtk_page_setup_new_from_file: %s: error", file);
+		g_free(file);
+		return FALSE;
+	}
+	g_free(file);
+
+	if (page_setup)
+		g_object_unref(page_setup);
+	page_setup = new_setup;
+
+	return TRUE;
+#else
+	return FALSE;
+#endif /* GTK_CHECK_VERSION(2, 12, 0) */
+}
+
+static gboolean printing_save_page_setup(void)
+{
+#if GTK_CHECK_VERSION(2, 12, 0)
+	gchar *file;
+	gboolean ret = TRUE;
+
+	g_return_val_if_fail(page_setup != NULL, FALSE);
+
+	debug_print("printing_save_page_setup: save page setup\n");
+
+	file = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, "pagesetuprc", NULL);
+	if (gtk_page_setup_to_file(page_setup, file, NULL) != TRUE) {
+		g_warning("printing_save_page_setup: gtk_page_setup_to_file: %s: error", file);
+		ret = FALSE;
+	}
+	g_free(file);
+
+	return ret;
+#else
+	return FALSE;
+#endif /* GTK_CHECK_VERSION(2, 12, 0) */
+}
+
 gint printing_print_messages_gtk(GSList *mlist, MimeInfo *partinfo,
 				 gboolean all_headers)
 {
@@ -462,8 +578,12 @@ gint printing_print_messages_gtk(GSList *mlist, MimeInfo *partinfo,
 			 print_data);
 	g_signal_connect(op, "draw-page", G_CALLBACK(draw_page), print_data);
 
+	if (!settings)
+		printing_load_settings();
 	if (settings)
 		gtk_print_operation_set_print_settings(op, settings);
+	if (!page_setup)
+		printing_load_page_setup();
 	if (page_setup)
 		gtk_print_operation_set_default_page_setup(op, page_setup);
 
@@ -472,11 +592,10 @@ gint printing_print_messages_gtk(GSList *mlist, MimeInfo *partinfo,
 		 GTK_WINDOW(main_window_get()->window), NULL);
 
 	if (res == GTK_PRINT_OPERATION_RESULT_APPLY) {
-		g_print("save settings\n");
 		if (settings)
 			g_object_unref(settings);
-		settings = g_object_ref
-			(gtk_print_operation_get_print_settings(op));
+		settings = g_object_ref(gtk_print_operation_get_print_settings(op));
+		printing_save_settings();
 	}
 
 	g_object_unref(op);
@@ -490,6 +609,10 @@ gint printing_print_messages_gtk(GSList *mlist, MimeInfo *partinfo,
 		g_free(print_data->msgs[i].hdr_data);
 		if (print_data->msgs[i].fp)
 			fclose(print_data->msgs[i].fp);
+		if (print_data->msgs[i].msg_text_file) {
+			g_unlink(print_data->msgs[i].msg_text_file);
+			g_free(print_data->msgs[i].msg_text_file);
+		}
 	}
 	g_free(print_data);
 
@@ -505,16 +628,21 @@ void printing_page_setup_gtk(void)
 {
 	GtkPageSetup *new_page_setup;
 
-	if (settings == NULL)
+	if (!settings)
+		printing_load_settings();
+	if (!settings)
 		settings = gtk_print_settings_new();
+	if (!page_setup)
+		printing_load_page_setup();
 
 	new_page_setup = gtk_print_run_page_setup_dialog
 		(GTK_WINDOW(main_window_get()->window), page_setup, settings);
 
 	if (page_setup)
 		g_object_unref(page_setup);
-
 	page_setup = new_page_setup;
+
+	printing_save_page_setup();
 }
 
 #endif /* GTK_CHECK_VERSION(2, 10, 0) */
